// =========================================================================================================
// Rating Object - v2.0build20191010
// =========================================================================================================

function RatingSystem() {
	var g_stars = gdi.Image(imgPath + "misc\\stars.png");
	var g_tooltip = window.CreateTooltip("Segoe UI", Math.round(sysWidth / 100));
	var g_tt_txt = "Bad;Average;Good;Very Good;Excellent".split(";");
	var g_hover = g_active = false;

	var d_tfo = fb.TitleFormat("$min($if2(%rating%,0),5)");
	var m_tfo = fb.TitleFormat("$min($if2($meta(rating),0),5)");
	var r_tfo = fb.TitleFormat("$min($if2($meta(rating),$if2(%rating%,0)),5)");
	var p_tfo = fb.TitleFormat("$if2(%play_count%,0)");

	var d_mode = window.GetProperty("Display Mode", 0);
	var r_mode = window.GetProperty("Rating Mode", 0);
	var p_limit = window.GetProperty("Playcount limit", 50);

	this.curMetadb = null;
	this.curRating = -1;

	this.initPos = function(x, y, w) {
		this.left = x ? x : 0;
		this.top = y ? y : 0;
		this.w = w ? w : ww;
		this.rs = this.w / 5;
		this.right = this.left + this.w;
		this.bottom = this.top + this.rs;
	}

	this.calcAutoRating = function() {
		var a = p_tfo.Eval();
		var b = new Date();
		var c = a > 0 ? new Date(fb.TitleFormat("$date(%first_played%)").Eval()) : b;
		var d = a * 10 - Math.floor((b.getTime() - c.getTime()) / 86400000 / 7.3);
		this.arIdx = Math.min(d / 10, 5);
		return Math.max(Math.floor(this.arIdx), 0);
	}

	this.getPlayCounts = function() {
		return Math.max(Math.min(Math.floor(p_tfo.Eval() / (p_limit / 5)), 5), 0);
	}

	this.initRating = function() {
		var a = r_mode == 1 ? d_tfo : r_mode == 2 ? m_tfo : r_tfo;
		this.curRating = g_active ? (d_mode == 1 ? this.calcAutoRating() : d_mode == 2 ? this.getPlayCounts() : a.Eval()) - 1 : -1;
	}			

	this.draw = function(gr) {
		for (var i = 0; i < 5; i++) {
			var a = i > this.curRating ? 5 : g_hover ? 3 : d_mode;
			g_stars && gr.DrawImage(g_stars, this.left + i * this.rs, this.top, this.rs, this.rs, 48 * a, 0, 48, 48, 0, fb.IsPlaying ? 255 : 128);
		}
	}

	this.repaint = function() {
		window.RepaintRect(this.left, this.top, this.w, this.rs);
	}

	this.setMetaRating = function(d) {
		var b = this.curMetadb, c = fb.CreateHandleList();
		c.Add(b);
		c.UpdateFileInfoFromJSON(JSON.stringify({ "RATING" : d }));
		c.Dispose();
	}

	this.traceMouse = function(x, y) {
		return x >= this.left && x <= this.right && y >= this.top && y <= this.bottom;
	}

	this.mouseMove = function(x, y) {
		if (g_active && this.traceMouse(x, y)) {
			var a = Math.min(Math.floor((x - this.left) / this.rs), 5);
			if (this.curRating != a) {
				this.curRating = a;
				g_tooltip.Deactivate();
				if (d_mode == 0) {
					g_hover = true;
					g_tooltip.Text = g_tt_txt[this.curRating];
					window.SetCursor(32649);
					this.repaint();
				} else g_tooltip.Text = d_mode == 1 ? "Autorate-Index: " + this.arIdx : "File played " + p_tfo.Eval() + " times";
				g_tooltip.Activate();
 			}
		} else this.mouseLeave();
	}

	this.mouseLeave = function() {
		g_hover = false;
		this.initRating();
		this.repaint();
		window.SetCursor(32512);
		g_tooltip.Deactivate();
	}

	this.onClick = function(x, y) {
		if (g_active && this.traceMouse(x, y) && d_mode == 0 && this.curRating > -1) {
			r_mode != 1 && this.setMetaRating(this.curRating + 1);
			r_mode != 2 && fb.RunContextCommand("Playback Statistics/Rating/" + (this.curRating + 1));
		}
	}

	this.ratingMenu = function(x, y) {
		var a = window.CreatePopupMenu(), b = window.CreatePopupMenu(), c = r_mode == 1 ? d_tfo : r_mode == 2 ? m_tfo : r_tfo, d = [5, 10, 25, 50, 100, 250, 500, 1000], e, idx;

		d_mode == 0 && (a.AppendMenuItem(c.Eval() > 0 ? 0 : 1, 1, "Delete rating"), a.AppendMenuSeparator());
		a.AppendMenuItem(0, 2, "Show/enable rating");
		a.AppendMenuItem(0, 3, "Show autorating");
		a.AppendMenuItem(0, 4, "Show playcounts");
		a.CheckMenuRadioItem(2, 4, d_mode + 2);

		if (d_mode == 0) {
			a.AppendMenuSeparator();
			a.AppendMenuItem(0, 5, "Rate to database/tags");
			a.AppendMenuItem(0, 6, "Rate to database only");
			a.AppendMenuItem(0, 7, "Rate to tags only");
			a.CheckMenuRadioItem(5, 7, r_mode + 5);
		}

		if (d_mode == 2) {
			a.AppendMenuSeparator();
			b.AppendTo(a, 0 | 16, "Playcount limit");
			for (var i = 0; i < 8; i++) {
				b.AppendMenuItem(0, 101 + i, d[i]);
				if (p_limit == d[i]) e = 101 + i;
			}
			b.CheckMenuRadioItem(101, 108, e);
		}

		idx = a.TrackPopupMenu(x, y);

		switch (true) {
			case idx == 0:
				break;

			case idx == 1:
				r_mode != 1 && this.setMetaRating("");
				r_mode != 2 && fb.RunContextCommand("Playback Statistics/Rating/<not set>");
				break;

			case idx < 5:
				window.SetProperty("Display Mode", idx - 2);
				d_mode = window.GetProperty("Display Mode");
				this.initRating();
				this.repaint();
				break;

			case idx < 8:
				window.SetProperty("Rating Mode", idx - 5);
				r_mode = window.GetProperty("Rating Mode");
				this.initRating();
				this.repaint();
				break;

			case idx >= 101 && idx <= 108:
				window.SetProperty("Playcount limit", d[idx - 101]);
				p_limit = window.GetProperty("Playcount limit");
				this.initRating();
				this.repaint();
				break;
		}
		a.Dispose();
		b.Dispose();
	}

	this.newTrack = function(metadb) {
		if (metadb) g_active = fb.PlaybackLength <= 0 || metadb.RawPath.indexOf("FOO_LASTFM" ) == 0 ? false : true;
		if (this.curMetadb = metadb) this.initRating();
	}

	this.onStop = function(reason) {
		reason != 2 && (g_active = false, this.initRating(), this.repaint());
	}

	this.newTrack(fb.GetNowPlaying());
}

var rating_system = new RatingSystem();